@@ -10,7 +10,6 @@ import csv
 from datetime import datetime
 import re
 import gffutils
-from biocommons.seqrepo import SeqRepo
 from gene.vrs_locations import SequenceLocation, ChromosomeLocation
 
 
@@ -33,43 +32,49 @@ class NCBI(Base):
         :param str data_dir: FTP data directory to use
         :param str assembly: The genome assembly
         """
-        self._database = database
+        super().__init__(database, host, data_dir)
         self._sequence_location = SequenceLocation()
         self._chromosome_location = ChromosomeLocation()
         self._data_url = f"ftp://{host}"
-        self._host = host
-        self._data_dir = data_dir
         self._assembly = assembly
+        self._date_today = datetime.today().strftime('%Y%m%d')
+
+    def perform_etl(self):
+        """Perform ETL methods.
+
+        :return: Concept IDs of concepts successfully loaded
+        """
         self._extract_data()
         self._transform_data()
+        self._database.flush_batch()
+        return self._processed_ids
 
     def _download_data(self, ncbi_dir: Path):
         """Download NCBI info, history, and GRCh38 files.
 
         :param str ncbi_dir: The NCBI data directory
         """
-        version = datetime.today().strftime('%Y%m%d')
-
         # Download info
         data_dir = f'{self._data_dir}GENE_INFO/Mammalia/'
-        fn = f'ncbi_info_{version}.tsv'
+        fn = f'ncbi_info_{self._date_today}.tsv'
         data_fn = 'Homo_sapiens.gene_info.gz'
         logger.info('Downloading NCBI gene_info....')
         self._ftp_download(self._host, data_dir, fn, ncbi_dir, data_fn)
         logger.info('Successfully downloaded NCBI gene_info.')
 
         # Download history
-        fn = f'ncbi_history_{version}.tsv'
+        fn = f'ncbi_history_{self._date_today}.tsv'
         data_fn = 'gene_history.gz'
         logger.info('Downloading NCBI gene_history...')
         self._ftp_download(self._host, self._data_dir, fn, ncbi_dir, data_fn)
         logger.info('Successfully downloaded NCBI gene_history.')
 
         # Download gff
+        og_fn = 'GCF_000001405.39_GRCh38.p13'
         data_dir = 'genomes/refseq/vertebrate_mammalian/Homo_sapiens/' \
-                   'latest_assembly_versions/GCF_000001405.39_GRCh38.p13/'
+                   f'latest_assembly_versions/{og_fn}/'
         fn = f'ncbi_{self._assembly}.gff'
-        data_fn = 'GCF_000001405.39_GRCh38.p13_genomic.gff.gz'
+        data_fn = f'{og_fn}_genomic.gff.gz'
         logger.info('Downloading NCBI gff data file...')
         self._ftp_download(self._host, data_dir, fn, ncbi_dir, data_fn)
         logger.info('Successfully downloaded NCBI gff data file.')
@@ -87,11 +92,11 @@ class NCBI(Base):
         gff_downloaded: bool = False
 
         for f in files:
-            if f.name.startswith('ncbi_info'):
+            if f.name.startswith(f'ncbi_info_{self._date_today}'):
                 info_downloaded = True
-            elif f.name.startswith('ncbi_history'):
+            elif f.name.startswith(f'ncbi_history_{self._date_today}'):
                 history_downloaded = True
-            elif f.name.startswith('ncbi_GRCh38'):
+            elif f.name.startswith('ncbi_GRCh38.p13'):
                 gff_downloaded = True
         return info_downloaded and history_downloaded and gff_downloaded
 
@@ -509,8 +514,7 @@ class NCBI(Base):
         prev_symbols = self._get_prev_symbols()
         info_genes = self._get_gene_info(prev_symbols)
 
-        seqrepo_dir = PROJECT_ROOT / 'data' / 'seqrepo' / 'latest'
-        sr = SeqRepo(seqrepo_dir)
+        sr = self.get_seqrepo()
 
         # create db for gff file
         db = gffutils.create_db(str(self._gff_src),
